@@ -1,4 +1,4 @@
-/* Copyright 2016 Torbjørn Birch Moltu
+/* Copyright 2016 The encode_unicode Developers
  *
  * Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
  * http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
@@ -7,13 +7,14 @@
  */
 
 
-//! Boilerplatey error enums
+//! Boilerplatey error types
 
 extern crate core;
 use self::core::fmt::{self,Display,Formatter};
 #[cfg(feature="std")]
 use std::error::Error;
 
+
 macro_rules! description {($err:ty, $desc:expr) => {
     #[cfg(not(feature="std"))]
     impl $err {
@@ -36,11 +37,40 @@ macro_rules! description {($err:ty, $desc:expr) => {
 }}
 
 
-/// Cannot tell whether an `u16` needs an extra unit,
-/// because it's a trailing surrogate itself.
-#[derive(Clone,Copy, Debug, PartialEq,Eq)]
-pub struct InvalidUtf16FirstUnit;
-description!{InvalidUtf16FirstUnit, |_| "Is a trailing surrogate" }
+macro_rules! single_cause {(#[$doc1:meta] #[$doc2:meta] $err:ident => $desc:expr) => {
+    // Rust 1.15 doesn't understand $(#[$doc:meta])* $:ident
+    #[$doc1]
+    #[$doc2]
+    #[derive(Clone,Copy, Debug, PartialEq,Eq)]
+    pub struct $err;
+    description!{$err, |_| $desc }
+}}
+
+
+single_cause!{
+    /// Cannot tell whether an `u16` needs an extra unit,
+    /// because it's a trailing surrogate itself.
+    InvalidUtf16FirstUnit => "is a trailing surrogate"
+}
+
+single_cause!{
+    /// Cannot create an `Utf8Char` or `Utf16Char` from the first codepoint of a str,
+    /// because there are none.
+    EmptyStrError => "is empty"
+}
+
+single_cause!{
+    /// Cannot create an `Utf8Char` from a standalone `u8`
+    /// that is not an ASCII character.
+    NonAsciiError => "is not an ASCII character"
+}
+
+single_cause!{
+    /// Cannot create an `Utf16Char` from a standalone `u16` that is not a
+    /// codepoint in the basic multilingual plane, but part of a suurrogate pair.
+    NonBMPError => "is not a codepoint in the basic multilingual plane"
+}
+
 
 
 macro_rules! simple {(#[$tydoc:meta] $err:ident  {
@@ -88,6 +118,7 @@ simple!{/// Reasons why one or two `u16`s are not valid UTF-16, in sinking prece
         ::InvalidSecond => "the required second unit is not a trailing / low surrogate",
     }}
 
+
 simple!{/// Reasons why a slice of `u16`s doesn't start with valid UTF-16.
     InvalidUtf16Slice {
         /// The slice is empty.
@@ -101,14 +132,15 @@ simple!{/// Reasons why a slice of `u16`s doesn't start with valid UTF-16.
     }}
 
 
-simple!{/// Reasons why `Utf8Char::from_str()` failed.
+simple!{/// Reasons why `Utf8Char::from_str()` or `Utf16Char::from_str()` failed.
     FromStrError {
-        /// `Utf8Char` cannot store more than a single codepoint.
+        /// `Utf8Char` or `Utf16Char` cannot store more than a single codepoint.
         ::MultipleCodepoints => "has more than one codepoint",
-        /// `Utf8Char` cannot be empty.
+        /// `Utf8Char` or `Utf16Char` cannot be empty.
         ::Empty => "is empty",
     }}
 
+
 simple!{/// Reasons why a byte is not the start of a UTF-8 codepoint.
     InvalidUtf8FirstByte {
         /// Sequences cannot be longer than 4 bytes. Is given for values >= 240.
@@ -168,7 +200,7 @@ macro_rules! complex {
 pub enum InvalidUtf8 {
     /// Something is wrong with the first byte.
     FirstByte(InvalidUtf8FirstByte),
-    /// Thee byte at index 1...3 should be a continuation byte,
+    /// The byte at index 1...3 should be a continuation byte,
     /// but dosesn't fit the pattern 0b10xx_xxxx.
     NotAContinuationByte(usize),
     /// There are too many leading zeros: it could be a byte shorter.
@@ -229,7 +261,7 @@ complex!{InvalidUtf8Slice {
     } {
         InvalidUtf8Slice::Utf8(_) => "the sequence is invalid UTF-8",
         InvalidUtf8Slice::Codepoint(_) => "the encoded codepoint is invalid",
-        InvalidUtf8Slice::TooShort(0) => "the slice is empty",
+        InvalidUtf8Slice::TooShort(1) => "the slice is empty",
         InvalidUtf8Slice::TooShort(_) => "the slice is shorter than the sequence",
     } => true => {
         InvalidUtf8Slice::Utf8(ref u) => Some(u),
