@@ -1,4 +1,4 @@
-/* Copyright 2016 Torbjørn Birch Moltu
+/* Copyright 2016 The encode_unicode Developers
  *
  * Licensed under the Apache License, Version 2.0, <LICENSE-APACHE or
  * http://apache.org/licenses/LICENSE-2.0> or the MIT license <LICENSE-MIT or
@@ -49,24 +49,27 @@ older versions might work now but can break with a minor update.
 mod errors;
 mod traits;
 mod utf8_char;
-mod utf8_iterator;
+mod utf8_iterators;
 mod utf16_char;
-mod utf16_iterator;
+mod utf16_iterators;
 
-pub use traits::CharExt;
+pub use traits::{CharExt, U8UtfExt, U16UtfExt, StrExt, IterExt};
 pub use utf8_char::Utf8Char;
 pub use utf16_char::Utf16Char;
-pub use utf8_iterator::Utf8Iterator;
-pub use utf16_iterator::Utf16Iterator;
-pub use traits::U8UtfExt;
-pub use traits::U16UtfExt;
+pub use utf8_iterators::{Utf8Iterator, iter_bytes};
+pub use utf16_iterators::{Utf16Iterator, iter_units};
 
 pub mod error {// keeping the public interface in one file
     //! Errors returned by various conversion methods in this crate.
-    pub use utf8_char::FromStrError;
-    pub use errors::{InvalidCodepoint};
-    pub use errors::{InvalidUtf8};
+    pub use errors::{FromStrError, EmptyStrError};
+    pub use errors::{InvalidCodepoint, InvalidUtf8};
     pub use errors::{InvalidUtf8FirstByte,InvalidUtf16FirstUnit};
     pub use errors::{InvalidUtf8Slice,InvalidUtf16Slice};
     pub use errors::{InvalidUtf8Array,InvalidUtf16Tuple};
 }
+
+pub mod iterator {
+    //! Iterator types that you should rarely need to name
+    pub use utf8_iterators::{Utf8Iterator, Utf8CharSplitter, Utf8Chars, Utf8CharIndices};
+    pub use utf16_iterators::{Utf16Iterator, Utf16CharSplitter, Utf16Chars, Utf16CharIndices};
+}
